
/* Source code by Paul B Mann with tabs = 3 */

#include "calc_main.h"
#include "calc_parser.h"
#include "calc_actions.h"

#ifdef TOKEN_ACTIONS

///////////////////////////////////////////////////////////////////////////////
//                                                                           // 
//    error     																				  //

int   calc_token_action::error (short t)                    
{
	// Illegal character?
		if (token.end == token.start) 
		{
			token.end++;
			return (t); // Return zero (t should be zero).
		}
	// Do a symbol table lookup.
	// Could be a valid keyword or constant from an ambiguous grammar.
      return (lookup (t)); 
}

///////////////////////////////////////////////////////////////////////////////
//                                                                           // 
//    lookup    																				  //

int   calc_token_action::lookup (short t) // Look for Symbol.
{
		token.sti = add_symbol (t, token.start, token.end); 
      return (symbol[token.sti].term); // This works for {typedef}
}

#endif
#ifdef PARSE_ACTIONS

///////////////////////////////////////////////////////////////////////////////
//                                                                           // 
//    goal_

short calc_parse_action::goal_ (short p)                    
{
	//	int a = pact_arg [p]; // Get first parse action argument.
 		return (0);
}

///////////////////////////////////////////////////////////////////////////////
//                                                                           // 
//    defterm   																				  //

short calc_parse_action::defterm (short p) 
{
	   short i = pact_arg[p];				   // Get first argument index.
      short sti = PS [arg_numb[i]-1].sti; // Get sti for first argument.
      symbol[sti].term = arg_numb [i+1];  // Set term to second argument.
      return (0);
}

///////////////////////////////////////////////////////////////////////////////
//                                                                           // 
//    delete_tree																				  //

short calc_parse_action::delete_tree (short p)
{
      n_nodes--;                          // Get last node created.
      while (node[n_nodes].child > 0)     // While not leaf.      
      {
         n_nodes = node[n_nodes].child;   // Down to first node.  
      }                                 
      if (n_nodes == 1) root = 1;         // If AST is gone.     
      PS->node = 0;                       // Set *PS to 0.        
      return (0);
}  

#endif
#ifdef NODE_ACTIONS

///////////////////////////////////////////////////////////////////////////////
//                                                                           // 

short calc_node_action::add_ (int n)
{
		switch (status)
		{
			case TOP_DOWN:	 break;
			case PASS_OVER: break;
			case BOTTOM_UP: emitstr (n, "\t\tADD\n"); break;
		}
		return (0);
}

///////////////////////////////////////////////////////////////////////////////
//                                                                           // 

short calc_node_action::div_ (int n)
{
		switch (status)
		{
			case TOP_DOWN:	 break;
			case PASS_OVER: break;
			case BOTTOM_UP: emitstr (n, "\t\tDIV\n"); break;
		}
		return (0);
}

///////////////////////////////////////////////////////////////////////////////
//                                                                           // 

short calc_node_action::then_ (int n)
{												
		switch (status)
		{
			case TOP_DOWN:	 emitstr (n, "\t\tBR NZ endif&1\nthen&1:\n"); break;						
			case PASS_OVER: break;
			case BOTTOM_UP: break;
		}
		return (0);
}

///////////////////////////////////////////////////////////////////////////////
//                                                                           // 

short calc_node_action::else_ (int n)
{
		switch (status)
		{
			case TOP_DOWN:	 emitstr (n, "\t\tBR Z endif&1 \nelse&1:\n"); break;
			case PASS_OVER: break;
			case BOTTOM_UP: break;
		}
		return (0);
}

///////////////////////////////////////////////////////////////////////////////
//                                                                           // 

short calc_node_action::then2_ (int n)
{												
		switch (status)
		{
			case TOP_DOWN:	 emitstr (n, "\t\tBR NZ else&1 \nthen&1:\n"); break;
			case PASS_OVER: break;
			case BOTTOM_UP: break;
		}
		return (0);
}

///////////////////////////////////////////////////////////////////////////////
//                                                                           // 

short calc_node_action::else2_ (int n)
{
		switch (status)
		{
			case TOP_DOWN:	 emitstr (n, "\t\tBR endif&1   \nelse&1:\n"); break;
			case PASS_OVER: break;
			case BOTTOM_UP: break;
		}
		return (0);
}

///////////////////////////////////////////////////////////////////////////////
//                                                                           // 

short calc_node_action::eq_ (int n)
{
		switch (status)
		{
			case TOP_DOWN:	 break;
			case PASS_OVER: break;
			case BOTTOM_UP: emitstr (n, "\t\tEQ\n"); break;
		}
		return (0);
}

///////////////////////////////////////////////////////////////////////////////
//                                                                           // 

short calc_node_action::ne_ (int n)
{
		switch (status)
		{
			case TOP_DOWN:	 break;
			case PASS_OVER: break;
			case BOTTOM_UP: emitstr (n, "\t\tNE\n"); break;
		}
		return (0);
}

///////////////////////////////////////////////////////////////////////////////
//                                                                           // 

short calc_node_action::goal_ (int n)
{
	//	int a = nact_arg [node[n].prod]; // Get first node action argument.
		switch (status)
		{
			case TOP_DOWN:	 emitstr (n, "\t\tSTART\n"); break;
			case PASS_OVER: break;
			case BOTTOM_UP: emitstr (n, "\t\tEOF\n"); break;
		}
      return (0);
}

///////////////////////////////////////////////////////////////////////////////
//                                                                           // 

short calc_node_action::ident_ (int n)
{
		switch (status)
		{
			case TOP_DOWN:	 break;
			case PASS_OVER: break;
			case BOTTOM_UP: emitstr (n, "\t\tLOAD %s\n"); break;
		}
		return (0);
}

///////////////////////////////////////////////////////////////////////////////
//                                                                           // 

short calc_node_action::if_ (int n)
{
		switch (status)
		{
			case TOP_DOWN:	 emitstr (n, "if&0:\n"); break;
			case PASS_OVER: break;
			case BOTTOM_UP: emitstr (n, "endif&0:\n"); break;
		}
		return (0);
}

///////////////////////////////////////////////////////////////////////////////
//                                                                           // 

short calc_node_action::int_ (int n)
{
		switch (status)
		{
			case TOP_DOWN:	 break;
			case PASS_OVER: break;
			case BOTTOM_UP: emitstr (n, "\t\tLOAD %s\n"); break;
		}
		return (0);
}

///////////////////////////////////////////////////////////////////////////////
//                                                                           // 

short calc_node_action::mul_ (int n)
{
		switch (status)
		{
			case TOP_DOWN:	 break;
			case PASS_OVER: break;
			case BOTTOM_UP: emitstr (n, "\t\tMUL\n"); break;
		}
		return (0);
}

///////////////////////////////////////////////////////////////////////////////
//                                                                           // 

short calc_node_action::program_ (int n)
{
		switch (status)
		{
			case TOP_DOWN:	 emitstr (n, "\t\tPROGRAM %s\n");break;
			case PASS_OVER: break;
			case BOTTOM_UP: emitstr (n, "\t\tEND PROGRAM %s\n"); break;
		}
		return (0);
}

///////////////////////////////////////////////////////////////////////////////
//                                                                           // 

short calc_node_action::store_ (int n)
{
		switch (status)
		{
			case TOP_DOWN:	 break;
			case PASS_OVER: break;
			case BOTTOM_UP: emitstr (n, "\t\tSTORE\n"); break;
		}
		return (0);
}

///////////////////////////////////////////////////////////////////////////////
//                                                                           // 

short calc_node_action::sub_ (int n)
{
		switch (status)
		{
			case TOP_DOWN:	 break;
			case PASS_OVER: break;
			case BOTTOM_UP: emitstr (n, "\t\tSUB\n"); break;
		}
		return (0);
}

///////////////////////////////////////////////////////////////////////////////
//                                                                           // 

short calc_node_action::target_ (int n)
{
		switch (status)
		{
			case TOP_DOWN:	 break;
			case PASS_OVER: break;
			case BOTTOM_UP: emitstr (n, "\t\tLADR %s\n"); break;
		}
		return (0);
}

///////////////////////////////////////////////////////////////////////////////
//                                                                           // 

short calc_node_action::emit (int n) 
{
      int a;
   // Get string for this status (0 = TOP_DOWN, 1 = PASS_OVER, 2 = BOTTOM_UP).
      a = nact_arg [node[n].prod] + status;
      if (a >= 0 && arg_numb[a] >= 0)
      {
			emitstr (n, arg_text [arg_numb[a]]);
      }
		return (0);
}

#endif

/*--- End of Actions. -------------------------------------------------------*/

























